﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Security.Cryptography;
using ClientServer.Db;
using ClientServer.Helpers;
using ClientServer.Models;
using ClientServer.Models.CustomProjects;
using ClientServer.Models.Exercises;
using ClientServer.Models.Exercises.Release;
using ClientServer.Models.Exercises.Solution;
using ClientServer.Models.Exercises.Tests;
using ClientServer.Models.Files;
using ClientServer.Models.Users;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;

namespace ClientServer.Controllers.Core
{
    [Route(Constants.InitControllerAbsolutePath)]
    public class Init : Controller
    {
        private readonly YapexDbContext _context;


        public Init(YapexDbContext context)
        {
            _context = context;
        }

        [HttpGet]
        public void Get()
        {
            //check if at least 1 user exists... 

            var firstUser = _context.Users.FirstOrDefault();

            if (firstUser != null)
            {
                //at least on user exits --> do not insert basic data twice
                return;
            }
            else
            {
                //else we need to create some basic data
                // e.g. first user, test types plangs, langs, ... 
            }

            Console.WriteLine("Creating first login (id: j, pw: j), CHANGE ADMIN CREDENTIALS AFTER LOGIN");

            //TODO remove

            //var blog = new Blog()
            //{
            //    Name = "BLOG",
            //    Posts = new List<Post>()
            //};

            //var post1 = new Post()
            //{
            //    Title = "t1",
            //};

            //var post2 = new Post()
            //{
            //    Title = "t2"
            //};

            //_context.Blogs.Add(blog);
            //_context.SaveChanges();

            //blog.Posts.Add(post1);
            //blog.Posts.Add(post2);
            //blog.MainPost = post1;


            //_context.SaveChanges();


            //one admin
            //one default group with role for all new users (students)
            //one default role for group creators

            #region set up first group independent 

            var adminRolePermission = new SystemRolePermission()
            {
                CanChangeRoles = true,
                CanCreateGroups = true,
                CanCreateRoles = true,
                CanDeleteGroups = true,
                CanDeleteRoles = true,
                CanChangeOtherUsersSystemRole = true,
                CanManageNewUsers = true,
                CanDeleteActivatedUsers = true,
                CanChangeSystemSettings = true,
                CanChangeUserData = true,
                CanManageTags = true,
                CanViewDashboard = true,
            };

            //the first system role cannot be changed/deleted!
            var adminSystemRole = new SystemRole()
            {
                DisplayName = Constants.DefaultSystemRoleDisplayName,
                Email = "admins@admins.de",
                SystemRolePermission = adminRolePermission
            };

            #endregion

            var adminPw = PasswordHelper.GetHash(PasswordHelper.HashLikeFrontend("j"));

            var admin = new User()
            {
                FirstName = "Admin",
                LastName = "Admin",
                Email = "admin@example.de",
                IsActivated = true,
                Token = "j", //admin //ahbxf
                Password = adminPw,
                SystemRole = adminSystemRole,
            };

            #region create first real group default group for all new users (students)

            var studentGroupRolePermission = new GroupRolePermission(false, false, false, false, false, false, false,
                false, false, false, false, false
            );

            var studentGrouRole = new GroupRole()
            {
                Id = InitialSystemSettings._DefaultGroupRoleId, //i don't think we can force this id but we can try...
                DisplayName = Constants.DefaultGroupRoleDisplayName,
                GroupRolePermission = studentGroupRolePermission
            };

            _context.GroupRoles.Add(studentGrouRole);

            //default group for all new users
            var studentGroup = new UserGroup()
            {
                Id = InitialSystemSettings._DefaultUserGroupId, //i don't think we can force this id but we can try...
                Email = "default@example.com",
                DisplayName = Constants.DefaultGroupDisplayName,
            };

            _context.UserGroups.Add(studentGroup);

            #endregion

            #region create creator (of groups) role and permission

            var creatorGroupRolePermission = new GroupRolePermission(true, true, true, true, true, true, true,
                true, true, true, true, true
            );

            var creatorGroupRole = new GroupRole()
            {
                Id = InitialSystemSettings
                    ._DefaultGroupCreatorGroupRoleId, //i don't think we can force this id but we can try...
                DisplayName = Constants.GroupCreatorRoleDisplayName,
                GroupRolePermission = creatorGroupRolePermission,
            };

            //the admin is the "creator" of the students group 
            var firstUserConnection = new UserWithUserGroup()
            {
                User = admin,
                UserGroup = studentGroup,
                GroupRole = creatorGroupRole,
            };

            _context.UserWithUserGroups.Add(firstUserConnection);

            #endregion

            #region add some programming langs

            //var csLang = new PLang()
            //{
            //    DisplayName = "C#",
            //    InternalName = "csharp"
            //};

            //_context.PLangs.Add(csLang);


            var javaLang = new PLang()
            {
                DisplayName = "Java",
                InternalName = "java",
                FileExtensionsWithDot = ".java",
                EditorHighlightModeName = "java"
            };
            _context.PLangs.Add(javaLang);


            //var pyLang = new PLang()
            //{
            //    DisplayName = "Python",
            //    InternalName = "python"
            //};

            //_context.PLangs.Add(pyLang);

            _context.Langs.Add(new Lang()
            {
                Language = "English",
                LangShortcut = "en"
            });

            _context.Langs.Add(new Lang()
            {
                Language = "Deutsch",
                LangShortcut = "de"
            });

            #endregion

            var blackBoxTest = new TestType()
            {
                DisplayName = "Black-Box-Test",
                InternalName = Constants.BlackBoxTestInternalName
            };
            var syntaxTest = new TestType()
            {
                DisplayName = "Syntaxtest",
                InternalName = Constants.CompileTestInternalName
            };
            var regexTest = new TestType()
            {
                DisplayName = "Regex-Test",
                InternalName = Constants.RegexTestInternalName
            };
            var compareFilesTest = new TestType()
            {
                DisplayName = "Dateienvergleich-Test",
                InternalName = Constants.CompareFilesTestInternalName
            };

            _context.TestTypes.Add(blackBoxTest);
            _context.TestTypes.Add(syntaxTest);
            _context.TestTypes.Add(regexTest);
            _context.TestTypes.Add(compareFilesTest);

            var systemSettings = new SystemSetting()
            {
                DefaultGroupCreatorGroupRole = creatorGroupRole,
                DefaultGroupRole = studentGrouRole,
                DefaultUserGroup = studentGroup,
                MaxCustomProjectsPerUser = InitialSystemSettings.MaxCustomProjectsPerUser,
                MaxCustomTestsPerParticipation = InitialSystemSettings.MaxCustomTestsPerParticipation,
                CurrentServerMessage = InitialSystemSettings.CurrentServerMessage,
                TestServerStatsUrl = InitialSystemSettings.TestServerStatsUrl,
                SubmitTestServerUrl = InitialSystemSettings.SubmitTestServerUrl,
                TestServerUrl = InitialSystemSettings.TestServerUrl,
                TestServerConfigUiUrl = InitialSystemSettings.TestServerConfigUiUrl,
                SubmitTestServerTimeoutInMs = InitialSystemSettings.SubmitTestServerTimeoutInMs,
                TestServerTimeoutInMs = InitialSystemSettings.TestServerTimeoutInMs,
                MaxNumberOfTestsWithOneRequest = InitialSystemSettings.MaxNumberOfTestsWithOneRequest,
                MaxNumberOfTestsWithOneRequestSubmitTestServer =
                    InitialSystemSettings.MaxNumberOfTestsWithOneRequestSubmitTestServer,
                JustRunProgramTimeoutInMs = InitialSystemSettings.JustRunProgramTimeoutInMs,
                JustRunProgramCompileTimeoutInMs = InitialSystemSettings.JustRunProgramCompileTimeoutInMs,
                JustRunProgramMemoryLimitInKb = InitialSystemSettings.JustRunProgramMemoryLimitInKb,
                JustRunProgramMaxDiskSpaceInKb = InitialSystemSettings.JustRunProgramMaxDiskSpaceInKb,
                CustomProjectTestTimeoutInMs = InitialSystemSettings.CustomProjectTestTimeoutInMs,
                CustomProjectTestCompileTimeoutInMs = InitialSystemSettings.CustomProjectTestCompileTimeoutInMs,
                CustomProjectTestMemoryLimitInKb = InitialSystemSettings.CustomProjectTestMemoryLimitInKb,
                CustomProjectTestMaxDiskSpaceInKb = InitialSystemSettings.CustomProjectTestMaxDiskSpaceInKb,
            };


            _context.SystemSettings.Add(systemSettings);

            #region test add inactive users

            //for (int i = 0; i < 5; i++)
            //{
            //    var newUser = new User()
            //    {
            //        Email = "example@example.de",
            //        IsActivated = false,
            //        FirstName = "Test " + (i + 1),
            //        LastName = "Tester" + (i + 1),
            //        Password = PasswordHelper.GetHash((i + 1) + ""),
            //        Token = "j" + (i + 1),
            //        UserWithUserGroups = new List<UserWithUserGroup>()
            //        {
            //            new UserWithUserGroup()
            //            {
            //                UserGroup = systemSettings.DefaultUserGroup,
            //                GroupRole = systemSettings.DefaultGroupRole
            //            }
            //        }
            //    };
            //    _context.Users.Add(newUser);
            //}

            #endregion

            int changed = _context.SaveChanges();


            //create example exercise

            Response.WriteAsync("OK -" + changed);
        }


//        [HttpGet("migrate/markdown")]
//        public void MigrateMarkdownFiles()
//        {
//            var basePath = Files.GetUploadFilePath(UploadDirType.MarkdownAssets);
//
//            var exDescriptions = _context.ExerciseDescriptions
//                    .Include(p => p.Assets)
//                    .Where(p => p.Assets.Count > 0)
//                    .ToList()
//                ;
//
//            int count = 0;
//            foreach (var exerciseDescription in exDescriptions)
//            {
//                foreach (var descriptionAsset in exerciseDescription.Assets)
//                {
//                    //create the real file
//
//                    try
//                    {
//                        using (var transaction = _context.Database.BeginTransaction())
//                        {
//                            try
//                            {
//                                var fileRef = new FileReferenceMarkdownAsset()
//                                {
//                                    OriginalName = descriptionAsset.DisplayName,
//                                    CreatedAt = (DateTime) _context.Entry(descriptionAsset)
//                                        .Property(YapexDbContext.CreatedAt)
//                                        .CurrentValue,
//                                    LastUpdatedAt = (DateTime) _context.Entry(descriptionAsset)
//                                        .Property(YapexDbContext.LastUpdatedAt).CurrentValue,
//                                    MimeType = descriptionAsset.MimeType,
//                                    SizeInBytes = descriptionAsset.Content.Length,
//                                    Hash = "",
//                                };
//
//                                var conn = new ExerciseDescriptionWithFileAsAssetReference()
//                                {
//                                    ExerciseDescription = exerciseDescription,
//                                    FileReferenceMarkdownAsset = fileRef
//                                };
//
//                                _context.ExerciseDescriptionWithFileAsAssetReferences.Add(conn);
//                                _context.SaveChanges();
//
//                                if (fileRef.Id <= 0)
//                                {
//                                    throw new Exception("fileRef must be tracked in db context");
//                                }
//
//                                try
//                                {
//                                    var info = new FileInfo(Path.Combine(basePath, fileRef.Id.ToString()));
//
//                                    if (info.Exists)
//                                    {
//                                        throw new Exception("file already exists");
//                                    }
//
//                                    using (var stream = new FileStream(info.FullName, FileMode.Create))
//                                    {
//                                        stream.Write(descriptionAsset.Content, 0, descriptionAsset.Content.Length);
//
//                                        using (var md5 = MD5.Create())
//                                        {
//                                            var result = md5.ComputeHash(descriptionAsset.Content);
//                                            fileRef.Hash = String.Join(String.Empty,
//                                                result.Select(p => p.ToString("x2")));
//                                        }
//                                    }
//                                }
//                                catch (Exception e)
//                                {
//                                    Console.WriteLine(
//                                        $"[ERROR] error creating file name: {descriptionAsset.DisplayName}");
//                                    Console.WriteLine(e.Message);
//                                    throw;
//                                }
//
//
//                                _context.SaveChanges();
//                                transaction.Commit();
//                            }
//                            catch (Exception e)
//                            {
//                                transaction.Rollback();
//                                Console.WriteLine(e);
//                                throw;
//                            }
//                        }
//
//                        count++;
//                    }
//                    catch (Exception e)
//                    {
//                        Console.WriteLine(e);
//                        throw;
//                    }
//                }
//            }
//
//
//            Response.WriteAsync("OK -" + count);
//        }
//
//
//        [HttpGet("migrate/ex/markdown")]
//        public void MigrateMarkdownExerciseDescriptionContent()
//        {
//            var allDescriptions = _context.ExerciseDescriptions
//                .Include(p => p.AssetReferences)
//                .ThenInclude(p => p.FileReferenceMarkdownAsset)
//                .Include(p => p.Assets)
//                .ToList();
//
//
//            int count = 0;
//            int count2 = 0;
//
//            using (var transaction = _context.Database.BeginTransaction())
//            {
//                try
//                {
//                    foreach (var exerciseDescription in allDescriptions)
//                    {
//                        if (exerciseDescription.Content.Contains(Constants.MarkdownAssetFileUrlPrefix))
//                        {
//
//                            foreach (var asset in exerciseDescription.Assets)
//                            {
//                                //find asset ref
//
//                                var assetRef = exerciseDescription.AssetReferences.FirstOrDefault(p =>
//                                    p.FileReferenceMarkdownAsset.OriginalName == asset.DisplayName &&
//                                    p.FileReferenceMarkdownAsset.SizeInBytes == asset.Content.Length);
//
//                                if (assetRef == null)
//                                {
//                                    throw new Exception($"could not find asset ref for asset id {asset.Id} for ex desc {exerciseDescription.Id}");
//                                }
//
//                                exerciseDescription.Content = exerciseDescription.Content.Replace(asset.AssetUrl,
//                                    Constants.MarkdownAssetFileUrlPrefix + assetRef.FileReferenceMarkdownAssetId);
//                                count2++;
//                            }
//                    
//                            count++;
//
//                        }
//                    }
//
//                    _context.SaveChanges();
//                    transaction.Commit();
//                }
//                catch (Exception e)
//                {
//                    transaction.Rollback();
//                    Console.WriteLine(e);
//                    throw;
//                }
//            }
//            
//            
//            Response.WriteAsync($"OK - {count} - ${count2}");
//        }
//
//
//        [HttpGet("migrate/tests")]
//        public void MigrateTestFiles()
//        {
//            var basePath = Files.GetUploadFilePath(UploadDirType.TestAssets);
//
//            var testsWithAssets = _context.Tests
//                    .Include(p => p.TestAssets)
//                    .Where(p => p.TestAssets.Count > 0)
//                    .ToList()
//                ;
//
//            int count = 0;
//            foreach (var testWithAsset in testsWithAssets)
//            {
//                foreach (var testAsset in testWithAsset.TestAssets)
//                {
//                    //create the real file
//
//                    try
//                    {
//                        using (var transaction = _context.Database.BeginTransaction())
//                        {
//                            try
//                            {
//                                var fileRef = new FileReferenceTestAsset()
//                                {
//                                    OriginalName = testAsset.DisplayName,
//                                    CreatedAt = (DateTime) _context.Entry(testAsset)
//                                        .Property(YapexDbContext.CreatedAt)
//                                        .CurrentValue,
//                                    LastUpdatedAt = (DateTime) _context.Entry(testAsset)
//                                        .Property(YapexDbContext.LastUpdatedAt).CurrentValue,
//                                    MimeType = testAsset.MimeType,
//                                    SizeInBytes = testAsset.Content.Length,
//                                    Hash = "",
//                                };
//
//                                var conn = new TestWithFileAsAssetReference()
//                                {
//                                    Test = testWithAsset,
//                                    FileReferenceTestAsset = fileRef
//                                };
//
//                                _context.TestWithFileAsAssetReferences.Add(conn);
//                                _context.SaveChanges();
//
//                                if (fileRef.Id <= 0)
//                                {
//                                    throw new Exception("fileRef must be tracked in db context");
//                                }
//
//                                try
//                                {
//                                    var info = new FileInfo(Path.Combine(basePath, fileRef.Id.ToString()));
//
//                                    if (info.Exists)
//                                    {
//                                        throw new Exception($"file already exists: {info.FullName}");
//                                    }
//
//                                    using (var stream = new FileStream(info.FullName, FileMode.Create))
//                                    {
//                                        stream.Write(testAsset.Content, 0, testAsset.Content.Length);
//
//                                        using (var md5 = MD5.Create())
//                                        {
//                                            var result = md5.ComputeHash(testAsset.Content);
//                                            fileRef.Hash = String.Join(String.Empty,
//                                                result.Select(p => p.ToString("x2")));
//                                        }
//                                    }
//                                }
//                                catch (Exception e)
//                                {
//                                    Console.WriteLine(
//                                        $"[ERROR] error creating file name: {testAsset.DisplayName}");
//                                    Console.WriteLine(e.Message);
//                                    throw;
//                                }
//
//
//                                _context.SaveChanges();
//                                transaction.Commit();
//                            }
//                            catch (Exception e)
//                            {
//                                transaction.Rollback();
//                                Console.WriteLine(e);
//                                throw;
//                            }
//                        }
//
//                        count++;
//                    }
//                    catch (Exception e)
//                    {
//                        Console.WriteLine(e);
//                        throw;
//                    }
//                }
//            }
//
//
//            Response.WriteAsync("OK -" + count);
//        }
//
//
//        [HttpGet("migrate/custom/tests")]
//        public void MigrateCustomTestAssets()
//        {
//            var basePath = Files.GetUploadFilePath(UploadDirType.UserAssets);
//
//            var testsWithAssets = _context.CustomTest
//                    .Include(p => p.TestAssets)
//                    .Where(p => p.TestAssets.Count > 0)
//                    .ToList()
//                ;
//
//            int count = 0;
//
//            foreach (var customTest in testsWithAssets)
//            {
//                foreach (var customTestAsset in customTest.TestAssets)
//                {
//                    //create the real file
//
//                    try
//                    {
//                        using (var transaction = _context.Database.BeginTransaction())
//                        {
//                            try
//                            {
//                                var fileRef = new FileReferenceUserFileAsset()
//                                {
//                                    OriginalName = customTestAsset.DisplayName,
//                                    CreatedAt = (DateTime) _context.Entry(customTestAsset)
//                                        .Property(YapexDbContext.CreatedAt)
//                                        .CurrentValue,
//                                    LastUpdatedAt = (DateTime) _context.Entry(customTestAsset)
//                                        .Property(YapexDbContext.LastUpdatedAt).CurrentValue,
//                                    MimeType = customTestAsset.MimeType,
//                                    SizeInBytes = customTestAsset.Content.Length,
//                                    Hash = "",
//                                };
//
//                                var conn = new CustomTestWithFileAsAssetReference()
//                                {
//                                    CustomTest = customTest,
//                                    FileReferenceUserFileAsset = fileRef
//                                };
//
//                                _context.CustomTestWithFileAsAssetReferences.Add(conn);
//                                _context.SaveChanges();
//
//                                if (fileRef.Id <= 0)
//                                {
//                                    throw new Exception("fileRef must be tracked in db context");
//                                }
//
//                                try
//                                {
//                                    var info = new FileInfo(Path.Combine(basePath, fileRef.Id.ToString()));
//
//                                    if (info.Exists)
//                                    {
//                                        throw new Exception($"file already exists: {info.FullName}");
//                                    }
//
//                                    using (var stream = new FileStream(info.FullName, FileMode.Create))
//                                    {
//                                        stream.Write(customTestAsset.Content, 0, customTestAsset.Content.Length);
//
//                                        using (var md5 = MD5.Create())
//                                        {
//                                            var result = md5.ComputeHash(customTestAsset.Content);
//                                            fileRef.Hash = String.Join(String.Empty,
//                                                result.Select(p => p.ToString("x2")));
//                                        }
//                                    }
//                                }
//                                catch (Exception e)
//                                {
//                                    Console.WriteLine(
//                                        $"[ERROR] error creating file name: {customTestAsset.DisplayName}");
//                                    Console.WriteLine(e.Message);
//                                    throw;
//                                }
//
//
//                                _context.SaveChanges();
//                                transaction.Commit();
//                            }
//                            catch (Exception e)
//                            {
//                                transaction.Rollback();
//                                Console.WriteLine(e);
//                                throw;
//                            }
//                        }
//
//                        count++;
//                    }
//                    catch (Exception e)
//                    {
//                        Console.WriteLine(e);
//                        throw;
//                    }
//                }
//                
//            }
//            
//            Response.WriteAsync("OK -" + count);
//            
//        }
//        
//        
//        [HttpGet("migrate/custom/project/description")]
//        public void MigrateCustomProjectDescriptionAssets()
//        {
//            var basePath = Files.GetUploadFilePath(UploadDirType.UserAssets);
//
//            var customProjectDescriptions = _context.CustomProjectDescriptions
//                    .Include(p => p.Assets)
//                    .Where(p => p.Assets.Count > 0)
//                    .ToList()
//                ;
//
//            int count = 0;
//
//            foreach (var customProjectDescription in customProjectDescriptions)
//            {
//                foreach (var customAsset in customProjectDescription.Assets)
//                {
//                    //create the real file
//
//                    try
//                    {
//                        using (var transaction = _context.Database.BeginTransaction())
//                        {
//                            try
//                            {
//                                var fileRef = new FileReferenceUserFileAsset()
//                                {
//                                    OriginalName = customAsset.DisplayName,
//                                    CreatedAt = (DateTime) _context.Entry(customAsset)
//                                        .Property(YapexDbContext.CreatedAt)
//                                        .CurrentValue,
//                                    LastUpdatedAt = (DateTime) _context.Entry(customAsset)
//                                        .Property(YapexDbContext.LastUpdatedAt).CurrentValue,
//                                    MimeType = customAsset.MimeType,
//                                    SizeInBytes = customAsset.Content.Length,
//                                    Hash = "",
//                                };
//
//                                var conn = new CustomProjectDescriptionWithFileAsAssetReference()
//                                {
//                                    CustomProjectDescription = customProjectDescription,
//                                    FileReferenceUserFileAsset = fileRef
//                                };
//
//                                _context.CustomProjectDescriptionWithFileAsAssetReferences.Add(conn);
//                                _context.SaveChanges();
//
//                                if (fileRef.Id <= 0)
//                                {
//                                    throw new Exception("fileRef must be tracked in db context");
//                                }
//
//                                try
//                                {
//                                    var info = new FileInfo(Path.Combine(basePath, fileRef.Id.ToString()));
//
//                                    if (info.Exists)
//                                    {
//                                        throw new Exception($"file already exists: {info.FullName}");
//                                    }
//
//                                    using (var stream = new FileStream(info.FullName, FileMode.Create))
//                                    {
//                                        stream.Write(customAsset.Content, 0, customAsset.Content.Length);
//
//                                        using (var md5 = MD5.Create())
//                                        {
//                                            var result = md5.ComputeHash(customAsset.Content);
//                                            fileRef.Hash = String.Join(String.Empty,
//                                                result.Select(p => p.ToString("x2")));
//                                        }
//                                    }
//                                }
//                                catch (Exception e)
//                                {
//                                    Console.WriteLine(
//                                        $"[ERROR] error creating file name: {customAsset.DisplayName}");
//                                    Console.WriteLine(e.Message);
//                                    throw;
//                                }
//
//
//                                _context.SaveChanges();
//                                transaction.Commit();
//                            }
//                            catch (Exception e)
//                            {
//                                transaction.Rollback();
//                                Console.WriteLine(e);
//                                throw;
//                            }
//                        }
//
//                        count++;
//                    }
//                    catch (Exception e)
//                    {
//                        Console.WriteLine(e);
//                        throw;
//                    }
//                }
//                
//            }
//            
//            Response.WriteAsync("OK -" + count);
//            
//        }
//        
//        [HttpGet("migrate/custom/project/markdown")]
//        public void MigrateMarkdownCustomProjectDescriptionContent()
//        {
//            
//            var allDescriptions = _context.CustomProjectDescriptions
//                .Include(p => p.AssetReferences)
//                .ThenInclude(p => p.FileReferenceUserFileAsset)
//                .Include(p => p.Assets)
//                .ToList();
//
//
//            int count = 0;
//            int count2 = 0;
//
//            using (var transaction = _context.Database.BeginTransaction())
//            {
//                try
//                {
//                    foreach (var customProjectDescription in allDescriptions)
//                    {
//                        if (customProjectDescription.Content.Contains(Constants.MarkdownAssetFileUrlPrefix))
//                        {
//
//                            foreach (var asset in customProjectDescription.Assets)
//                            {
//                                //find asset ref
//
//                                var assetRef = customProjectDescription.AssetReferences.FirstOrDefault(p =>
//                                    p.FileReferenceUserFileAsset.OriginalName == asset.DisplayName &&
//                                    p.FileReferenceUserFileAsset.SizeInBytes == asset.Content.Length);
//
//                                if (assetRef == null)
//                                {
//                                    throw new Exception($"could not find asset ref for asset id {asset.Id} for ex desc {customProjectDescription.Id}");
//                                }
//
//                                customProjectDescription.Content = customProjectDescription.Content.Replace(asset.AssetUrl,
//                                    Constants.MarkdownAssetFileUrlPrefix + assetRef.FileReferenceUserFileAssetId);
//                                count2++;
//                            }
//                    
//                            count++;
//
//                        }
//                    }
//
//                    _context.SaveChanges();
//                    transaction.Commit();
//                }
//                catch (Exception e)
//                {
//                    transaction.Rollback();
//                    Console.WriteLine(e);
//                    throw;
//                }
//            }
//            
//            
//            Response.WriteAsync($"OK - {count} - ${count2}");
//        }
//        
//        [HttpGet("migrate/custom/project/tests")]
//        public void MigrateCustomProjectTestAssets()
//        {
//            var basePath = Files.GetUploadFilePath(UploadDirType.UserAssets);
//
//            var testsWithAssets = _context.CustomProjectTests
//                    .Include(p => p.TestAssets)
//                    .Where(p => p.TestAssets.Count > 0)
//                    .ToList()
//                ;
//
//            int count = 0;
//
//            foreach (var customProjectTest in testsWithAssets)
//            {
//                foreach (var customProjectTestAsset in customProjectTest.TestAssets)
//                {
//                    //create the real file
//
//                    try
//                    {
//                        using (var transaction = _context.Database.BeginTransaction())
//                        {
//                            try
//                            {
//                                var fileRef = new FileReferenceUserFileAsset()
//                                {
//                                    OriginalName = customProjectTestAsset.DisplayName,
//                                    CreatedAt = (DateTime) _context.Entry(customProjectTestAsset)
//                                        .Property(YapexDbContext.CreatedAt)
//                                        .CurrentValue,
//                                    LastUpdatedAt = (DateTime) _context.Entry(customProjectTestAsset)
//                                        .Property(YapexDbContext.LastUpdatedAt).CurrentValue,
//                                    MimeType = customProjectTestAsset.MimeType,
//                                    SizeInBytes = customProjectTestAsset.Content.Length,
//                                    Hash = "",
//                                };
//
//                                var conn = new CustomProjectTestWithFileAsAssetReference()
//                                {
//                                    CustomProjectTest = customProjectTest,
//                                    FileReferenceUserFileAsset = fileRef
//                                };
//
//                                _context.CustomProjectTestWithFileAsAssetReferences.Add(conn);
//                                _context.SaveChanges();
//
//                                if (fileRef.Id <= 0)
//                                {
//                                    throw new Exception("fileRef must be tracked in db context");
//                                }
//
//                                try
//                                {
//                                    var info = new FileInfo(Path.Combine(basePath, fileRef.Id.ToString()));
//
//                                    if (info.Exists)
//                                    {
//                                        throw new Exception($"file already exists: {info.FullName}");
//                                    }
//
//                                    using (var stream = new FileStream(info.FullName, FileMode.Create))
//                                    {
//                                        stream.Write(customProjectTestAsset.Content, 0, customProjectTestAsset.Content.Length);
//
//                                        using (var md5 = MD5.Create())
//                                        {
//                                            var result = md5.ComputeHash(customProjectTestAsset.Content);
//                                            fileRef.Hash = String.Join(String.Empty,
//                                                result.Select(p => p.ToString("x2")));
//                                        }
//                                    }
//                                }
//                                catch (Exception e)
//                                {
//                                    Console.WriteLine(
//                                        $"[ERROR] error creating file name: {customProjectTestAsset.DisplayName}");
//                                    Console.WriteLine(e.Message);
//                                    throw;
//                                }
//
//
//                                _context.SaveChanges();
//                                transaction.Commit();
//                            }
//                            catch (Exception e)
//                            {
//                                transaction.Rollback();
//                                Console.WriteLine(e);
//                                throw;
//                            }
//                        }
//
//                        count++;
//                    }
//                    catch (Exception e)
//                    {
//                        Console.WriteLine(e);
//                        throw;
//                    }
//                }
//                
//            }
//            
//            Response.WriteAsync("OK -" + count);
//            
//        }
        
    }
}
